@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2010 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -226,17 +226,16 @@ char *get_env_var(request_rec *r, char *name) {
 }
 
 /**
- * Internal log helper function. Use msr_log instead. This function will
- * correctly handle both the messages that have a newline at the end, and
- * those that don't.
+ * Extended internal log helper function. Use msr_log instead. If fixup is
+ * true, the message will be stripped of any trailing newline and any
+ * required bytes will be escaped.
  */
-void internal_log(request_rec *r, directory_config *dcfg, modsec_rec *msr,
-    int level, const char *text, va_list ap)
+void internal_log_ex(request_rec *r, directory_config *dcfg, modsec_rec *msr,
+    int level, int fixup, const char *text, va_list ap)
 {
     apr_size_t nbytes, nbytes_written;
     apr_file_t *debuglog_fd = NULL;
     int filter_debug_level = 0;
-    int str2len;
     char str1[1024] = "";
     char str2[1256] = "";
 
@@ -257,19 +256,26 @@ void internal_log(request_rec *r, directory_config *dcfg, modsec_rec *msr,
      */
     if ((level > 3)&&( (debuglog_fd == NULL) || (level > filter_debug_level) )) return;
 
-    /* Construct the message (leaving a byte left for a newline if needed). */
+    /* Construct the message. */
     apr_vsnprintf(str1, sizeof(str1), text, ap);
-    str2len = apr_snprintf(str2, sizeof(str2) - 1, 
-        "[%s] [%s/sid#%pp][rid#%pp][%s][%d] %s",
+    if (fixup) {
+        int len = strlen(str1);
+
+        /* Strip line ending. */
+        if (len && str1[len - 1] == '\n') {
+            str1[len - 1] = '\0';
+        }
+        if (len > 1 && str1[len - 2] == '\r') {
+            str1[len - 2] = '\0';
+        }
+    }
+
+    /* Construct the log entry. */
+    apr_snprintf(str2, sizeof(str2), 
+        "[%s] [%s/sid#%pp][rid#%pp][%s][%d] %s\n",
         current_logtime(msr->mp), ap_get_server_name(r), (r->server),
         r, ((r->uri == NULL) ? "" : log_escape_nq(msr->mp, r->uri)),
-        level, str1);
-
-    /* Add a newline if there is not one already (needed for msr_log_*) */
-    if (str2[str2len - 1] != '\n') {
-        str2[str2len] = '\n';
-        str2[str2len + 1] = '\0';
-    }
+        level, (fixup ? log_escape_nq(msr->mp, str1) : str1));
 
     /* Write to the debug log. */
     if ((debuglog_fd != NULL)&&(level <= filter_debug_level)) {
@@ -277,7 +283,8 @@ void internal_log(request_rec *r, directory_config *dcfg, modsec_rec *msr,
         apr_file_write_full(debuglog_fd, str2, nbytes, &nbytes_written);
     }
 
-    /* Send message levels 1-3 to the Apache error log too. */
+    /* Send message levels 1-3 to the Apache error log and 
+     * add it to the message list in the audit log. */
     if (level <= 3) {
         char *unique_id = (char *)get_env_var(r, "UNIQUE_ID");
         char *hostname = (char *)msr->hostname;
@@ -310,6 +317,15 @@ void internal_log(request_rec *r, directory_config *dcfg, modsec_rec *msr,
     return;
 }
 
+/**
+ * Internal log helper function. Use msr_log instead.
+ */
+void internal_log(request_rec *r, directory_config *dcfg, modsec_rec *msr,
+    int level, const char *text, va_list ap)
+{
+    internal_log_ex(r, dcfg, msr, level, 0, text, ap);
+}
+
 /**
  * Logs one message at the given level to the debug log (and to the
  * Apache error log if the message is important enough.
@@ -318,7 +334,7 @@ void msr_log(modsec_rec *msr, int level, const char *text, ...) {
     va_list ap;
 
     va_start(ap, text);
-    internal_log(msr->r, msr->txcfg, msr, level, text, ap);
+    internal_log_ex(msr->r, msr->txcfg, msr, level, 0, text, ap);
     va_end(ap);
 }
 
@@ -328,24 +344,24 @@ void msr_log(modsec_rec *msr, int level, const char *text, ...) {
  * Apache error log. This is intended for error callbacks.
  */
 void msr_log_error(modsec_rec *msr, const char *text, ...) {
-    const char *str = text;
     va_list ap;
 
     va_start(ap, text);
-    internal_log(msr->r, msr->txcfg, msr, 3, str, ap);
+    internal_log_ex(msr->r, msr->txcfg, msr, 3, 1, text, ap);
     va_end(ap);
 }
 
 /**
  * Logs one message at level 4 to the debug log and to the
  * Apache error log. This is intended for warning callbacks.
+ *
+ * The 'text' will first be escaped.
  */
 void msr_log_warn(modsec_rec *msr, const char *text, ...) {
-    const char *str = text;
     va_list ap;
 
     va_start(ap, text);
-    internal_log(msr->r, msr->txcfg, msr, 4, str, ap);
+    internal_log_ex(msr->r, msr->txcfg, msr, 4, 1, text, ap);
     va_end(ap);
 }
 
