@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2010 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -19,6 +19,16 @@
 #ifndef _UTIL_H_
 #define _UTIL_H_
 
+#include <sys/types.h>
+#include <apr_file_info.h>
+
+#ifndef APR_WSTICKY
+/* Add extra flags added to APR in 0.9.5 */
+#define APR_USETID      0x8000 /**< Set user id */
+#define APR_GSETID      0x4000 /**< Set group id */
+#define APR_WSTICKY     0x2000 /**< Sticky bit */
+#endif
+
 #include "modsecurity.h"
 
 int DSOLOCAL normalise_path_inplace(unsigned char *input, int len, int win, int *changed);
@@ -53,6 +63,8 @@ char DSOLOCAL *current_logtime(apr_pool_t *mp);
 
 char DSOLOCAL *current_filetime(apr_pool_t *mp);
 
+int DSOLOCAL msc_mkstemp_ex(char *template, int mode);
+
 int DSOLOCAL msc_mkstemp(char *template);
 
 char DSOLOCAL *strtolower_inplace(unsigned char *str);
@@ -73,6 +85,8 @@ char DSOLOCAL *log_escape_hex(apr_pool_t *mp, const unsigned char *text, unsigne
 
 char DSOLOCAL *log_escape_raw(apr_pool_t *mp, const unsigned char *text, unsigned long int text_length);
 
+char DSOLOCAL *log_escape_nul(apr_pool_t *mp, const unsigned char *text, unsigned long int text_length);
+
 char DSOLOCAL *_log_escape(apr_pool_t *p, const unsigned char *input,
     unsigned long int input_length, int escape_quotes, int escape_colon);
 
@@ -94,4 +108,6 @@ char DSOLOCAL *resolve_relative_path(apr_pool_t *pool, const char *parent_filena
 
 int DSOLOCAL css_decode_inplace(unsigned char *input, long int input_len);
 
+apr_fileperms_t DSOLOCAL mode2fileperms(int mode);
+
 #endif
